@@ -3,19 +3,19 @@ use crate::{
     components::{
         event_pump, CommandBlocking, CommandInfo, CommitComponent,
         Component, DrawableComponent, HelpComponent, MsgComponent,
-        ResetComponent,
+        ResetComponent, StashMsgComponent,
     },
     keys,
-    queue::{InternalEvent, NeedsUpdate, Queue},
+    queue::{Action, InternalEvent, NeedsUpdate, Queue},
     strings,
-    tabs::{Revlog, Status},
+    tabs::{Revlog, StashList, Stashing, Status},
     ui::style::Theme,
 };
+use anyhow::{anyhow, Result};
 use asyncgit::{sync, AsyncNotification, CWD};
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
 use itertools::Itertools;
-use log::trace;
 use std::borrow::Cow;
 use strings::commands;
 use tui::{
@@ -26,7 +26,6 @@ use tui::{
     widgets::{Block, Borders, Paragraph, Tabs, Text},
     Frame,
 };
-
 ///
 pub struct App {
     do_quit: bool,
@@ -34,10 +33,13 @@ pub struct App {
     msg: MsgComponent,
     reset: ResetComponent,
     commit: CommitComponent,
+    stashmsg_popup: StashMsgComponent,
     current_commands: Vec<CommandInfo>,
     tab: usize,
     revlog: Revlog,
     status_tab: Status,
+    stashing_tab: Stashing,
+    stashlist_tab: StashList,
     queue: Queue,
     theme: Theme,
 }
@@ -53,6 +55,10 @@ impl App {
         Self {
             reset: ResetComponent::new(queue.clone(), &theme),
             commit: CommitComponent::new(queue.clone(), &theme),
+            stashmsg_popup: StashMsgComponent::new(
+                queue.clone(),
+                &theme,
+            ),
             do_quit: false,
             current_commands: Vec::new(),
             help: HelpComponent::new(&theme),
@@ -60,13 +66,18 @@ impl App {
             tab: 0,
             revlog: Revlog::new(&sender, &theme),
             status_tab: Status::new(&sender, &queue, &theme),
+            stashing_tab: Stashing::new(&sender, &queue, &theme),
+            stashlist_tab: StashList::new(&queue, &theme),
             queue,
             theme,
         }
     }
 
     ///
-    pub fn draw<B: Backend>(&mut self, f: &mut Frame<B>) {
+    pub fn draw<B: Backend>(
+        &mut self,
+        f: &mut Frame<B>,
+    ) -> Result<()> {
         let chunks_main = Layout::default()
             .direction(Direction::Vertical)
             .constraints(
@@ -81,11 +92,14 @@ impl App {
 
         self.draw_tabs(f, chunks_main[0]);
 
-        if self.tab == 0 {
-            self.status_tab.draw(f, chunks_main[1]);
-        } else {
-            self.revlog.draw(f, chunks_main[1]);
-        }
+        //TODO: macro because of generic draw call
+        match self.tab {
+            0 => self.status_tab.draw(f, chunks_main[1])?,
+            1 => self.revlog.draw(f, chunks_main[1])?,
+            2 => self.stashing_tab.draw(f, chunks_main[1])?,
+            3 => self.stashlist_tab.draw(f, chunks_main[1])?,
+            _ => return Err(anyhow!("unknown tab")),
+        };
 
         Self::draw_commands(
             f,
@@ -94,25 +108,27 @@ impl App {
             self.theme,
         );
 
-        self.draw_popups(f);
+        self.draw_popups(f)?;
+
+        Ok(())
     }
 
     ///
-    pub fn event(&mut self, ev: Event) {
-        trace!("event: {:?}", ev);
+    pub fn event(&mut self, ev: Event) -> Result<()> {
+        log::trace!("event: {:?}", ev);
 
         if self.check_quit(ev) {
-            return;
+            return Ok(());
         }
 
         let mut flags = NeedsUpdate::empty();
 
-        if event_pump(ev, self.components_mut().as_mut_slice()) {
+        if event_pump(ev, self.components_mut().as_mut_slice())? {
             flags.insert(NeedsUpdate::COMMANDS);
         } else if let Event::Key(k) = ev {
             let new_flags = match k {
                 keys::TAB_TOGGLE => {
-                    self.toggle_tabs();
+                    self.toggle_tabs()?;
                     NeedsUpdate::COMMANDS
                 }
 
@@ -122,39 +138,52 @@ impl App {
             flags.insert(new_flags);
         }
 
-        let new_flags = self.process_queue();
+        let new_flags = self.process_queue()?;
         flags.insert(new_flags);
 
         if flags.contains(NeedsUpdate::ALL) {
-            self.update();
+            self.update()?;
         }
         if flags.contains(NeedsUpdate::DIFF) {
-            self.status_tab.update_diff();
+            self.status_tab.update_diff()?;
         }
         if flags.contains(NeedsUpdate::COMMANDS) {
             self.update_commands();
         }
+
+        Ok(())
     }
 
     //TODO: do we need this?
-    ///
-    pub fn update(&mut self) {
-        trace!("update");
-        self.status_tab.update();
+    /// forward ticking to components that require it
+    pub fn update(&mut self) -> Result<()> {
+        log::trace!("update");
+        self.status_tab.update()?;
+        self.revlog.update()?;
+        self.stashing_tab.update()?;
+        self.stashlist_tab.update()?;
+
+        Ok(())
     }
 
     ///
-    pub fn update_git(&mut self, ev: AsyncNotification) {
-        trace!("update_git: {:?}", ev);
+    pub fn update_git(
+        &mut self,
+        ev: AsyncNotification,
+    ) -> Result<()> {
+        log::trace!("update_git: {:?}", ev);
 
-        self.status_tab.update_git(ev);
+        self.status_tab.update_git(ev)?;
+        self.stashing_tab.update_git(ev)?;
 
         match ev {
             AsyncNotification::Diff => (),
-            AsyncNotification::Log => self.revlog.update(),
+            AsyncNotification::Log => self.revlog.update()?,
             //TODO: is that needed?
             AsyncNotification::Status => self.update_commands(),
         }
+
+        Ok(())
     }
 
     ///
@@ -166,12 +195,26 @@ impl App {
     pub fn any_work_pending(&self) -> bool {
         self.status_tab.anything_pending()
             || self.revlog.any_work_pending()
+            || self.stashing_tab.anything_pending()
     }
 }
 
 // private impls
 impl App {
-    accessors!(self, [msg, reset, commit, help, revlog, status_tab]);
+    accessors!(
+        self,
+        [
+            msg,
+            reset,
+            commit,
+            stashmsg_popup,
+            help,
+            revlog,
+            status_tab,
+            stashing_tab,
+            stashlist_tab
+        ]
+    );
 
     fn check_quit(&mut self, ev: Event) -> bool {
         if let Event::Key(e) = ev {
@@ -183,17 +226,37 @@ impl App {
         false
     }
 
-    fn toggle_tabs(&mut self) {
-        self.tab += 1;
-        self.tab %= 2;
+    fn get_tabs(&mut self) -> Vec<&mut dyn Component> {
+        vec![
+            &mut self.status_tab,
+            &mut self.revlog,
+            &mut self.stashing_tab,
+            &mut self.stashlist_tab,
+        ]
+    }
 
-        if self.tab == 1 {
-            self.status_tab.hide();
-            self.revlog.show();
-        } else {
-            self.status_tab.show();
-            self.revlog.hide();
+    fn toggle_tabs(&mut self) -> Result<()> {
+        let mut new_tab = self.tab + 1;
+        {
+            let tabs = self.get_tabs();
+            new_tab %= tabs.len();
         }
+        self.set_tab(new_tab)
+    }
+
+    fn set_tab(&mut self, tab: usize) -> Result<()> {
+        let tabs = self.get_tabs();
+        for (i, t) in tabs.into_iter().enumerate() {
+            if tab == i {
+                t.show()?;
+            } else {
+                t.hide();
+            }
+        }
+
+        self.tab = tab;
+
+        Ok(())
     }
 
     fn update_commands(&mut self) {
@@ -202,48 +265,41 @@ impl App {
         self.current_commands.sort_by_key(|e| e.order);
     }
 
-    fn process_queue(&mut self) -> NeedsUpdate {
+    fn process_queue(&mut self) -> Result<NeedsUpdate> {
         let mut flags = NeedsUpdate::empty();
         loop {
             let front = self.queue.borrow_mut().pop_front();
             if let Some(e) = front {
-                flags.insert(self.process_internal_event(e));
+                flags.insert(self.process_internal_event(e)?);
             } else {
                 break;
             }
         }
         self.queue.borrow_mut().clear();
 
-        flags
+        Ok(flags)
     }
 
     fn process_internal_event(
         &mut self,
         ev: InternalEvent,
-    ) -> NeedsUpdate {
+    ) -> Result<NeedsUpdate> {
         let mut flags = NeedsUpdate::empty();
         match ev {
-            InternalEvent::ResetItem(reset_item) => {
-                if reset_item.is_folder {
-                    if sync::reset_workdir_folder(
-                        CWD,
-                        reset_item.path.as_str(),
-                    )
-                    .is_ok()
-                    {
+            InternalEvent::ConfirmedAction(action) => match action {
+                Action::Reset(r) => {
+                    if Status::reset(&r) {
                         flags.insert(NeedsUpdate::ALL);
                     }
-                } else if sync::reset_workdir_file(
-                    CWD,
-                    reset_item.path.as_str(),
-                )
-                .is_ok()
-                {
-                    flags.insert(NeedsUpdate::ALL);
                 }
-            }
-            InternalEvent::ConfirmResetItem(reset_item) => {
-                self.reset.open_for_path(reset_item);
+                Action::StashDrop(s) => {
+                    if StashList::drop(s) {
+                        flags.insert(NeedsUpdate::ALL);
+                    }
+                }
+            },
+            InternalEvent::ConfirmAction(action) => {
+                self.reset.open(action)?;
                 flags.insert(NeedsUpdate::COMMANDS);
             }
             InternalEvent::AddHunk(hash) => {
@@ -251,9 +307,7 @@ impl App {
                     self.status_tab.selected_path()
                 {
                     if is_stage {
-                        if sync::unstage_hunk(CWD, path, hash)
-                            .unwrap()
-                        {
+                        if sync::unstage_hunk(CWD, path, hash)? {
                             flags.insert(NeedsUpdate::ALL);
                         }
                     } else if sync::stage_hunk(CWD, path, hash)
@@ -264,14 +318,20 @@ impl App {
                 }
             }
             InternalEvent::ShowErrorMsg(msg) => {
-                self.msg.show_msg(msg.as_str());
-                flags.insert(NeedsUpdate::ALL);
+                self.msg.show_msg(msg.as_str())?;
+                flags
+                    .insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS);
             }
             InternalEvent::Update(u) => flags.insert(u),
-            InternalEvent::OpenCommit => self.commit.show(),
+            InternalEvent::OpenCommit => self.commit.show()?,
+            InternalEvent::PopupStashing(opts) => {
+                self.stashmsg_popup.options(opts);
+                self.stashmsg_popup.show()?
+            }
+            InternalEvent::TabSwitch => self.set_tab(0)?,
         };
 
-        flags
+        Ok(flags)
     }
 
     fn commands(&self, force_all: bool) -> Vec<CommandInfo> {
@@ -314,20 +374,32 @@ impl App {
             || self.msg.is_visible()
     }
 
-    fn draw_popups<B: Backend>(&mut self, f: &mut Frame<B>) {
+    fn draw_popups<B: Backend>(
+        &mut self,
+        f: &mut Frame<B>,
+    ) -> Result<()> {
         let size = f.size();
 
-        self.commit.draw(f, size);
-        self.reset.draw(f, size);
-        self.help.draw(f, size);
-        self.msg.draw(f, size);
+        self.commit.draw(f, size)?;
+        self.stashmsg_popup.draw(f, size)?;
+        self.reset.draw(f, size)?;
+        self.help.draw(f, size)?;
+        self.msg.draw(f, size)?;
+
+        Ok(())
     }
 
+    //TODO: make this dynamic
     fn draw_tabs<B: Backend>(&self, f: &mut Frame<B>, r: Rect) {
         f.render_widget(
             Tabs::default()
                 .block(Block::default().borders(Borders::BOTTOM))
-                .titles(&[strings::TAB_STATUS, strings::TAB_LOG])
+                .titles(&[
+                    strings::TAB_STATUS,
+                    strings::TAB_LOG,
+                    strings::TAB_STASHING,
+                    "Stashes",
+                ])
                 .style(Style::default())
                 .highlight_style(
                     self.theme
