@@ -1,6 +1,11 @@
 #![forbid(unsafe_code)]
-// #![warn(clippy::cargo)]
+#![deny(clippy::cargo)]
+//TODO: remove once crossterm upgraded to current mio:
+//https://github.com/crossterm-rs/crossterm/issues/432
+#![allow(clippy::cargo::multiple_crate_versions)]
 #![deny(clippy::pedantic)]
+#![deny(clippy::result_unwrap_used)]
+#![deny(clippy::panic)]
 #![allow(clippy::module_name_repetitions)]
 
 mod app;
@@ -15,6 +20,7 @@ mod ui;
 mod version;
 
 use crate::{app::App, poll::QueueEvent};
+use anyhow::{anyhow, Result};
 use asyncgit::AsyncNotification;
 use backtrace::Backtrace;
 use crossbeam_channel::{tick, unbounded, Receiver, Select};
@@ -23,9 +29,8 @@ use crossterm::{
         disable_raw_mode, enable_raw_mode, EnterAlternateScreen,
         LeaveAlternateScreen,
     },
-    ExecutableCommand, Result,
+    ExecutableCommand,
 };
-use log::error;
 use scopeguard::defer;
 use scopetime::scope_time;
 use simplelog::{Config, LevelFilter, WriteLogger};
@@ -36,6 +41,7 @@ use std::{
     io::{self, Write},
     panic,
     path::PathBuf,
+    process,
     time::{Duration, Instant},
 };
 use tui::{
@@ -47,21 +53,19 @@ static TICK_INTERVAL: Duration = Duration::from_secs(5);
 static SPINNER_INTERVAL: Duration = Duration::from_millis(50);
 
 fn main() -> Result<()> {
-    setup_logging();
+    setup_logging()?;
 
     if invalid_path() {
         eprintln!("invalid git path\nplease run gitui inside of a git repository");
         return Ok(());
     }
 
-    enable_raw_mode()?;
-    io::stdout().execute(EnterAlternateScreen)?;
+    setup_terminal()?;
     defer! {
-        io::stdout().execute(LeaveAlternateScreen).unwrap();
-        disable_raw_mode().unwrap();
+        shutdown_terminal().expect("shutdown failed");
     }
 
-    set_panic_handlers();
+    set_panic_handlers()?;
 
     let mut terminal = start_terminal(io::stdout())?;
 
@@ -73,7 +77,7 @@ fn main() -> Result<()> {
     let ticker = tick(TICK_INTERVAL);
     let spinner_ticker = tick(SPINNER_INTERVAL);
 
-    app.update();
+    app.update()?;
     draw(&mut terminal, &mut app)?;
 
     let mut spinner = Spinner::default();
@@ -84,7 +88,7 @@ fn main() -> Result<()> {
             &rx_git,
             &ticker,
             &spinner_ticker,
-        );
+        )?;
 
         {
             scope_time!("loop");
@@ -93,9 +97,9 @@ fn main() -> Result<()> {
 
             for e in events {
                 match e {
-                    QueueEvent::InputEvent(ev) => app.event(ev),
-                    QueueEvent::Tick => app.update(),
-                    QueueEvent::GitEvent(ev) => app.update_git(ev),
+                    QueueEvent::InputEvent(ev) => app.event(ev)?,
+                    QueueEvent::Tick => app.update()?,
+                    QueueEvent::GitEvent(ev) => app.update_git(ev)?,
                     QueueEvent::SpinnerUpdate => {
                         needs_draw = false;
                         spinner.update()
@@ -118,11 +122,27 @@ fn main() -> Result<()> {
     Ok(())
 }
 
+fn setup_terminal() -> Result<()> {
+    enable_raw_mode()?;
+    io::stdout().execute(EnterAlternateScreen)?;
+    Ok(())
+}
+
+fn shutdown_terminal() -> Result<()> {
+    io::stdout().execute(LeaveAlternateScreen)?;
+    disable_raw_mode()?;
+    Ok(())
+}
+
 fn draw<B: Backend>(
     terminal: &mut Terminal<B>,
     app: &mut App,
 ) -> io::Result<()> {
-    terminal.draw(|mut f| app.draw(&mut f))
+    terminal.draw(|mut f| {
+        if let Err(e) = app.draw(&mut f) {
+            log::error!("failed to draw: {:?}", e)
+        }
+    })
 }
 
 fn invalid_path() -> bool {
@@ -134,7 +154,7 @@ fn select_event(
     rx_git: &Receiver<AsyncNotification>,
     rx_ticker: &Receiver<Instant>,
     rx_spinner: &Receiver<Instant>,
-) -> Vec<QueueEvent> {
+) -> Result<Vec<QueueEvent>> {
     let mut events: Vec<QueueEvent> = Vec::new();
 
     let mut sel = Select::new();
@@ -158,11 +178,10 @@ fn select_event(
         3 => oper
             .recv(rx_spinner)
             .map(|_| events.push(QueueEvent::SpinnerUpdate)),
-        _ => panic!("unknown select source"),
-    }
-    .unwrap();
+        _ => return Err(anyhow!("unknown select source")),
+    }?;
 
-    events
+    Ok(events)
 }
 
 fn start_terminal<W: Write>(
@@ -176,41 +195,51 @@ fn start_terminal<W: Write>(
     Ok(terminal)
 }
 
-#[must_use]
-pub fn get_app_config_path() -> PathBuf {
-    let mut path = dirs::cache_dir().unwrap();
+fn get_app_config_path() -> Result<PathBuf> {
+    let mut path = dirs::cache_dir()
+        .ok_or_else(|| anyhow!("failed to find os cache dir."))?;
+
     path.push("gitui");
-    fs::create_dir_all(&path).unwrap();
-    path
+    fs::create_dir_all(&path)?;
+    Ok(path)
 }
 
-fn setup_logging() {
+fn setup_logging() -> Result<()> {
     if env::var("GITUI_LOGGING").is_ok() {
-        let mut path = get_app_config_path();
+        let mut path = get_app_config_path()?;
         path.push("gitui.log");
 
         let _ = WriteLogger::init(
             LevelFilter::Trace,
             Config::default(),
-            File::create(path).unwrap(),
+            File::create(path)?,
         );
     }
+
+    Ok(())
 }
 
-fn set_panic_handlers() {
+fn set_panic_handlers() -> Result<()> {
     // regular panic handler
     panic::set_hook(Box::new(|e| {
         let backtrace = Backtrace::new();
-        error!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
+        log::error!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
+        shutdown_terminal().expect("shutdown failed inside panic");
+        eprintln!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
     }));
 
     // global threadpool
     rayon_core::ThreadPoolBuilder::new()
         .panic_handler(|e| {
-            error!("thread panic: {:?}", e);
-            panic!(e)
+            let backtrace = Backtrace::new();
+            log::error!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
+            shutdown_terminal()
+                .expect("shutdown failed inside panic");
+            eprintln!("panic: {:?}\ntrace:\n{:?}", e, backtrace);
+            process::abort();
         })
         .num_threads(4)
-        .build_global()
-        .unwrap();
+        .build_global()?;
+
+    Ok(())
 }
