@@ -1,3 +1,5 @@
+use crate::tabs::StashingOptions;
+use asyncgit::sync::CommitId;
 use bitflags::bitflags;
 use std::{cell::RefCell, collections::VecDeque, rc::Rc};
 
@@ -21,12 +23,18 @@ pub struct ResetItem {
     pub is_folder: bool,
 }
 
+///
+pub enum Action {
+    Reset(ResetItem),
+    StashDrop(CommitId),
+}
+
 ///
 pub enum InternalEvent {
     ///
-    ConfirmResetItem(ResetItem),
+    ConfirmAction(Action),
     ///
-    ResetItem(ResetItem),
+    ConfirmedAction(Action),
     ///
     AddHunk(u64),
     ///
@@ -35,6 +43,10 @@ pub enum InternalEvent {
     Update(NeedsUpdate),
     ///
     OpenCommit,
+    ///
+    PopupStashing(StashingOptions),
+    ///
+    TabSwitch,
 }
 
 ///
