@@ -3,6 +3,7 @@ pub static TITLE_DIFF: &str = "Diff: ";
 pub static TITLE_INDEX: &str = "Staged Changes [2]";
 
 pub static TAB_STATUS: &str = "Status";
+pub static TAB_STASHING: &str = "Stashing";
 pub static TAB_LOG: &str = "Log";
 pub static TAB_DIVIDER: &str = "  |  ";
 
@@ -11,11 +12,21 @@ pub static CMD_SPLITTER: &str = " ";
 pub static MSG_TITLE_ERROR: &str = "Error";
 pub static COMMIT_TITLE: &str = "Commit";
 pub static COMMIT_MSG: &str = "type commit message..";
-pub static RESET_TITLE: &str = "Reset";
-pub static RESET_MSG: &str = "confirm file reset?";
+pub static STASH_POPUP_TITLE: &str = "Stash";
+pub static STASH_POPUP_MSG: &str = "type name (optional)";
+pub static CONFIRM_TITLE_RESET: &str = "Reset";
+pub static CONFIRM_TITLE_STASHDROP: &str = "Drop";
+pub static CONFIRM_MSG_RESET: &str = "confirm file reset?";
+pub static CONFIRM_MSG_STASHDROP: &str = "confirm stash drop?";
+
+pub static LOG_TITLE: &str = "Commit";
+pub static STASHLIST_TITLE: &str = "Stashes";
 
 pub static HELP_TITLE: &str = "Help: all commands";
 
+pub static STASHING_FILES_TITLE: &str = "Files to Stash";
+pub static STASHING_OPTIONS_TITLE: &str = "Options";
+
 pub mod commands {
     use crate::components::CommandText;
 
@@ -23,6 +34,8 @@ pub mod commands {
     static CMD_GROUP_DIFF: &str = "Diff";
     static CMD_GROUP_CHANGES: &str = "Changes";
     static CMD_GROUP_COMMIT: &str = "Commit";
+    static CMD_GROUP_STASHING: &str = "Stashing";
+    static CMD_GROUP_STASHES: &str = "Stashes";
 
     ///
     pub static TOGGLE_TABS: CommandText = CommandText::new(
@@ -152,4 +165,43 @@ pub mod commands {
         "resets the file in question",
         CMD_GROUP_GENERAL,
     );
+
+    ///
+    pub static STASHING_SAVE: CommandText = CommandText::new(
+        "Save [s]",
+        "opens stash name input popup",
+        CMD_GROUP_STASHING,
+    );
+    ///
+    pub static STASHING_TOGGLE_INDEXED: CommandText =
+        CommandText::new(
+            "Toggle Staged [i]",
+            "toggle including staged files into stash",
+            CMD_GROUP_STASHING,
+        );
+    ///
+    pub static STASHING_TOGGLE_UNTRACKED: CommandText =
+        CommandText::new(
+            "Toggle Untracked [u]",
+            "toggle including untracked files into stash",
+            CMD_GROUP_STASHING,
+        );
+    ///
+    pub static STASHING_CONFIRM_MSG: CommandText = CommandText::new(
+        "Stash [enter]",
+        "save files to stash",
+        CMD_GROUP_STASHING,
+    );
+    ///
+    pub static STASHLIST_APPLY: CommandText = CommandText::new(
+        "Apply [enter]",
+        "apply selected stash",
+        CMD_GROUP_STASHES,
+    );
+    ///
+    pub static STASHLIST_DROP: CommandText = CommandText::new(
+        "Drop [D]",
+        "drop selected stash",
+        CMD_GROUP_STASHES,
+    );
 }
