@@ -1,19 +1,22 @@
 use crate::{
     accessors,
     components::{
-        event_pump, ChangesComponent, CommandBlocking, CommandInfo,
-        Component, DiffComponent, DrawableComponent,
+        self, event_pump, ChangesComponent, CommandBlocking,
+        CommandInfo, Component, DiffComponent, DrawableComponent,
         FileTreeItemKind,
     },
     keys,
-    queue::Queue,
+    queue::{Queue, ResetItem},
     strings,
     ui::style::Theme,
 };
+use anyhow::Result;
 use asyncgit::{
-    current_tick, AsyncDiff, AsyncNotification, AsyncStatus,
-    DiffParams,
+    sync::{self, status::StatusType},
+    AsyncDiff, AsyncNotification, AsyncStatus, DiffParams,
+    StatusParams, CWD,
 };
+use components::{command_pump, visibility_blocking};
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
 use strings::commands;
@@ -42,7 +45,8 @@ pub struct Status {
     index_wd: ChangesComponent,
     diff: DiffComponent,
     git_diff: AsyncDiff,
-    git_status: AsyncStatus,
+    git_status_workdir: AsyncStatus,
+    git_status_stage: AsyncStatus,
 }
 
 impl DrawableComponent for Status {
@@ -50,7 +54,7 @@ impl DrawableComponent for Status {
         &mut self,
         f: &mut tui::Frame<B>,
         rect: tui::layout::Rect,
-    ) {
+    ) -> Result<()> {
         let chunks = Layout::default()
             .direction(Direction::Horizontal)
             .constraints(
@@ -87,9 +91,11 @@ impl DrawableComponent for Status {
             )
             .split(chunks[0]);
 
-        self.index_wd.draw(f, left_chunks[0]);
-        self.index.draw(f, left_chunks[1]);
-        self.diff.draw(f, chunks[1]);
+        self.index_wd.draw(f, left_chunks[0])?;
+        self.index.draw(f, left_chunks[1])?;
+        self.diff.draw(f, chunks[1])?;
+
+        Ok(())
     }
 }
 
@@ -122,7 +128,8 @@ impl Status {
             ),
             diff: DiffComponent::new(queue.clone(), theme),
             git_diff: AsyncDiff::new(sender.clone()),
-            git_status: AsyncStatus::new(sender.clone()),
+            git_status_workdir: AsyncStatus::new(sender.clone()),
+            git_status_stage: AsyncStatus::new(sender.clone()),
         }
     }
 
@@ -134,7 +141,7 @@ impl Status {
         }
     }
 
-    fn switch_focus(&mut self, f: Focus) -> bool {
+    fn switch_focus(&mut self, f: Focus) -> Result<bool> {
         if self.focus != f {
             self.focus = f;
 
@@ -155,12 +162,12 @@ impl Status {
                 }
             };
 
-            self.update_diff();
+            self.update_diff()?;
 
-            return true;
+            return Ok(true);
         }
 
-        false
+        Ok(false)
     }
 
     fn set_diff_target(&mut self, target: DiffTarget) {
@@ -186,60 +193,90 @@ impl Status {
     }
 
     ///
-    pub fn update(&mut self) {
-        self.git_diff.refresh().unwrap();
-        self.git_status.fetch(current_tick()).unwrap();
+    pub fn update(&mut self) -> Result<()> {
+        if self.is_visible() {
+            self.git_diff.refresh()?;
+            self.git_status_workdir.fetch(StatusParams::new(
+                StatusType::WorkingDir,
+                true,
+            ))?;
+            self.git_status_stage
+                .fetch(StatusParams::new(StatusType::Stage, true))?;
+        }
+
+        Ok(())
     }
 
     ///
     pub fn anything_pending(&self) -> bool {
-        self.git_diff.is_pending() || self.git_status.is_pending()
+        self.git_diff.is_pending()
+            || self.git_status_stage.is_pending()
+            || self.git_status_workdir.is_pending()
     }
 
     ///
-    pub fn update_git(&mut self, ev: AsyncNotification) {
+    pub fn update_git(
+        &mut self,
+        ev: AsyncNotification,
+    ) -> Result<()> {
         match ev {
-            AsyncNotification::Diff => self.update_diff(),
-            AsyncNotification::Status => self.update_status(),
+            AsyncNotification::Diff => self.update_diff()?,
+            AsyncNotification::Status => self.update_status()?,
             _ => (),
         }
+
+        Ok(())
     }
 
-    fn update_status(&mut self) {
-        let status = self.git_status.last().unwrap();
-        self.index.update(&status.stage);
-        self.index_wd.update(&status.work_dir);
+    fn update_status(&mut self) -> Result<()> {
+        let status = self.git_status_stage.last()?;
+        self.index.update(&status.items)?;
 
-        self.update_diff();
+        let status = self.git_status_workdir.last()?;
+        self.index_wd.update(&status.items)?;
+
+        self.update_diff()?;
+
+        Ok(())
     }
 
     ///
-    pub fn update_diff(&mut self) {
+    pub fn update_diff(&mut self) -> Result<()> {
         if let Some((path, is_stage)) = self.selected_path() {
             let diff_params = DiffParams(path.clone(), is_stage);
 
             if self.diff.current() == (path.clone(), is_stage) {
                 // we are already showing a diff of the right file
                 // maybe the diff changed (outside file change)
-                if let Some((params, last)) =
-                    self.git_diff.last().unwrap()
-                {
+                if let Some((params, last)) = self.git_diff.last()? {
                     if params == diff_params {
-                        self.diff.update(path, is_stage, last);
+                        self.diff.update(path, is_stage, last)?;
                     }
                 }
             } else {
                 // we dont show the right diff right now, so we need to request
                 if let Some(diff) =
-                    self.git_diff.request(diff_params).unwrap()
+                    self.git_diff.request(diff_params)?
                 {
-                    self.diff.update(path, is_stage, diff);
+                    self.diff.update(path, is_stage, diff)?;
                 } else {
-                    self.diff.clear();
+                    self.diff.clear()?;
                 }
             }
         } else {
-            self.diff.clear();
+            self.diff.clear()?;
+        }
+
+        Ok(())
+    }
+
+    /// called after confirmation
+    pub fn reset(item: &ResetItem) -> bool {
+        if item.is_folder {
+            sync::reset_workdir_folder(CWD, item.path.as_str())
+                .is_ok()
+        } else {
+            sync::reset_workdir_file(CWD, item.path.as_str()).is_ok()
         }
     }
 }
@@ -250,75 +287,65 @@ impl Component for Status {
         out: &mut Vec<CommandInfo>,
         force_all: bool,
     ) -> CommandBlocking {
-        if self.visible {
-            for c in self.components() {
-                if c.commands(out, force_all)
-                    != CommandBlocking::PassingOn
-                    && !force_all
-                {
-                    break;
-                }
-            }
-
-            {
-                let focus_on_diff = self.focus == Focus::Diff;
-                out.push(CommandInfo::new(
-                    commands::STATUS_FOCUS_LEFT,
-                    true,
-                    (self.visible && focus_on_diff) || force_all,
-                ));
-                out.push(CommandInfo::new(
-                    commands::STATUS_FOCUS_RIGHT,
-                    self.can_focus_diff(),
-                    (self.visible && !focus_on_diff) || force_all,
-                ));
-            }
-
-            out.push(
-                CommandInfo::new(
-                    commands::SELECT_STATUS,
-                    true,
-                    (self.visible && self.focus == Focus::Diff)
-                        || force_all,
-                )
-                .hidden(),
-            );
-
-            out.push(
-                CommandInfo::new(
-                    commands::SELECT_STAGING,
-                    true,
-                    (self.visible && self.focus == Focus::WorkDir)
-                        || force_all,
-                )
-                .order(-2),
-            );
-
-            out.push(
-                CommandInfo::new(
-                    commands::SELECT_UNSTAGED,
-                    true,
-                    (self.visible && self.focus == Focus::Stage)
-                        || force_all,
-                )
-                .order(-2),
+        if self.visible || force_all {
+            command_pump(
+                out,
+                force_all,
+                self.components().as_slice(),
             );
         }
 
-        if self.visible {
-            CommandBlocking::Blocking
-        } else {
-            CommandBlocking::PassingOn
+        {
+            let focus_on_diff = self.focus == Focus::Diff;
+            out.push(CommandInfo::new(
+                commands::STATUS_FOCUS_LEFT,
+                true,
+                (self.visible && focus_on_diff) || force_all,
+            ));
+            out.push(CommandInfo::new(
+                commands::STATUS_FOCUS_RIGHT,
+                self.can_focus_diff(),
+                (self.visible && !focus_on_diff) || force_all,
+            ));
         }
+
+        out.push(
+            CommandInfo::new(
+                commands::SELECT_STATUS,
+                true,
+                (self.visible && self.focus == Focus::Diff)
+                    || force_all,
+            )
+            .hidden(),
+        );
+
+        out.push(
+            CommandInfo::new(
+                commands::SELECT_STAGING,
+                true,
+                (self.visible && self.focus == Focus::WorkDir)
+                    || force_all,
+            )
+            .order(-2),
+        );
+
+        out.push(
+            CommandInfo::new(
+                commands::SELECT_UNSTAGED,
+                true,
+                (self.visible && self.focus == Focus::Stage)
+                    || force_all,
+            )
+            .order(-2),
+        );
+
+        visibility_blocking(self)
     }
 
-    fn event(&mut self, ev: crossterm::event::Event) -> bool {
+    fn event(&mut self, ev: crossterm::event::Event) -> Result<bool> {
         if self.visible {
-            let conusmed =
-                event_pump(ev, self.components_mut().as_mut_slice());
-
-            if conusmed {
-                return true;
+            if event_pump(ev, self.components_mut().as_mut_slice())? {
+                return Ok(true);
             }
 
             if let Event::Key(k) = ev {
@@ -338,12 +365,12 @@ impl Component for Status {
                             DiffTarget::WorkingDir => Focus::WorkDir,
                         })
                     }
-                    _ => false,
+                    _ => Ok(false),
                 };
             }
         }
 
-        false
+        Ok(false)
     }
 
     fn is_visible(&self) -> bool {
@@ -354,7 +381,10 @@ impl Component for Status {
         self.visible = false;
     }
 
-    fn show(&mut self) {
+    fn show(&mut self) -> Result<()> {
         self.visible = true;
+        self.update()?;
+
+        Ok(())
     }
 }
